package com.its.api.its.service.vms;

import com.its.api.its.model.dto.vms.TbVmsFormDto;
import com.its.api.its.model.entity.vms.TbVmsForm;
import com.its.api.its.repository.vms.TbVmsFormRepository;
import com.its.api.utils.ItsUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Slf4j
@RequiredArgsConstructor
@Service
public class TbVmsFormService {

    private final TbVmsFormRepository repo;

    // 데이터 1건 조회, 없으면 exception
    private TbVmsForm requireOne(Integer id) {
        return this.repo.findById(id)
                .orElseThrow(() -> new NoSuchElementException("데이터가 존재하지 않습니다: " + id));
    }

    // 전체 데이터 조회
    @Transactional(readOnly = true)
    public List<TbVmsFormDto> findAll() {
        List<TbVmsFormDto> result = new ArrayList<>();
        List<TbVmsForm> data = this.repo.findAll();
        for (TbVmsForm entity : data) {
            result.add(entity.toDto());
        }
        return result;
    }

    /**
     * VMS TYPE(크기) 에 해당되는 폼 정보를 조회한다.
     * @param vmsTypeCd
     * @return
     */
    @Transactional(readOnly = true)
    public List<TbVmsFormDto> findAllByVmsType(String vmsTypeCd) {
        List<TbVmsFormDto> result = new ArrayList<>();
        List<TbVmsForm> data = this.repo.findAllByVmsType(vmsTypeCd);
        for (TbVmsForm entity : data) {
            TbVmsFormDto dto = entity.toDto();
            dto.setVmsFormImag(ItsUtils.convertBmpToPng(dto.getVmsFormImag()));
            dto.setImagFmt("png");
            result.add(dto);
        }
        return result;
    }

    /**
     * VMS TYPE(크기) 및 폼 유형에 해당되는 폼 정보를 조회한다.
     * @param vmsTypeCd
     * @param formTypeCd
     * @return
     */
    @Transactional(readOnly = true)
    public List<TbVmsFormDto> findAllByVmsTypeAndFormType(String vmsTypeCd, List<Integer> formTypeCd) {
        List<TbVmsFormDto> result = new ArrayList<>();
        List<TbVmsForm> data = this.repo.findAllByVmsTypeAndFormType(vmsTypeCd, formTypeCd);
        for (TbVmsForm entity : data) {
            TbVmsFormDto dto = entity.toDto();
            dto.setVmsFormImag(ItsUtils.convertBmpToPng(dto.getVmsFormImag()));
            dto.setImagFmt("png");
            result.add(dto);
        }
        return result;
    }
    // 교통정보
    public List<TbVmsFormDto> findAllTraffic(String vmsTypeCd) {
        List<Integer> formTypeCd = Arrays.asList(11, 12, 13, 14, 15);
        return findAllByVmsTypeAndFormType(vmsTypeCd, formTypeCd);
    }
    // 정체
    public List<TbVmsFormDto> findAllCongest(String vmsTypeCd) {
        List<Integer> formTypeCd = Arrays.asList(16);
        return findAllByVmsTypeAndFormType(vmsTypeCd, formTypeCd);
    }
    // 돌발
    public List<TbVmsFormDto> findAllIncident(String vmsTypeCd) {
        List<Integer> formTypeCd = Arrays.asList(20);
        return findAllByVmsTypeAndFormType(vmsTypeCd, formTypeCd);
    }
    // 공사/행사
    public List<TbVmsFormDto> findAllEvent(String vmsTypeCd) {
        List<Integer> formTypeCd = Arrays.asList(30);
        return findAllByVmsTypeAndFormType(vmsTypeCd, formTypeCd);
    }
    // 홍보
    public List<TbVmsFormDto> findAllHongbo(String vmsTypeCd) {
        List<Integer> formTypeCd = Arrays.asList(40);
        return findAllByVmsTypeAndFormType(vmsTypeCd, formTypeCd);
    }
    // 우회도로
    public List<TbVmsFormDto> findAllDetour(String vmsTypeCd) {
        List<Integer> formTypeCd = Arrays.asList(50);
        return findAllByVmsTypeAndFormType(vmsTypeCd, formTypeCd);
    }
    // 재난안전
    public List<TbVmsFormDto> findAllSafety(String vmsTypeCd) {
        List<Integer> formTypeCd = Arrays.asList(60);
        return findAllByVmsTypeAndFormType(vmsTypeCd, formTypeCd);
    }

    // 데이터 1건 조회(기존 데이터가 반드시 존재해야 함)
    @Transactional(readOnly = true)
    public TbVmsFormDto findById(Integer id) {
        TbVmsForm entity = requireOne(id);
        return entity.toDto();
    }

    // 데이터 변경
    @Transactional
    public TbVmsFormDto updateById(Integer id, TbVmsFormDto.TbVmsFormUpdReq req) {
        TbVmsForm entity = requireOne(id);
        entity.updateInfo(req);
        this.repo.save(entity);
        return entity.toDto();
    }

    // 데이터 변경 또는 생성-목록(데이터가 존재하면 업데이트 없으면 신규로 생성)
    @Transactional
    public List<TbVmsFormDto> mergeInfoList(List<TbVmsFormDto.TbVmsFormUpdReq> reqList) {
        List<TbVmsFormDto> result = new ArrayList<>();
        for (TbVmsFormDto.TbVmsFormUpdReq req : reqList) {
            TbVmsForm obj = req.toEntity();
            this.repo.save(obj);
            result.add(obj.toDto());
        }
        return result;
    }

    // 데이터 변경 또는 생성-개별(데이터가 존재하면 업데이트 없으면 신규로 생성)
    @Transactional
    public TbVmsFormDto mergeInfo(TbVmsFormDto.TbVmsFormUpdReq req) {
        TbVmsForm obj = req.toEntity();
        this.repo.save(obj);
        return obj.toDto();
    }

    // 정보 삭제-개별, 데이터 존재하지 않으면 Exception
    @Transactional
    public TbVmsFormDto deleteById(Integer id) {
        TbVmsForm entity = requireOne(id);
        this.repo.deleteById(id);
        return entity.toDto();
    }

    // 정보 삭제-목록, 존재하는 데이터 만 삭제
    @Transactional
    public List<TbVmsFormDto> deleteByIds(List<Integer> ids) {
        List<TbVmsFormDto> result = new ArrayList<>();
        for (Integer id : ids) {
            Optional<TbVmsForm> obj = this.repo.findById(id);
            if (obj.isPresent()) {
                this.repo.deleteById(id);
                result.add(obj.get().toDto());
            }
        }
        return result;
    }

}
